const fs = require('fs')
const handlebars = require('handlebars')
const querystring = require('querystring')
const Email = require('@pubsweet/component-send-email')
const config = require('config')

const resetPasswordPath = config.get('invite-reviewer.url')
const resetPath = config.get('invite-reset-password.url')

module.exports = {
  setupInviteEmail: async (user, emailType, inviteUrl) => {
    const replacements = {
      url: `${inviteUrl}${resetPath}?${querystring.encode({
        email: user.email,
        token: user.passwordResetToken,
        firstName: user.firstName,
        lastName: user.lastName,
        affiliation: user.affiliation,
        title: user.title,
      })}`,
    }

    const { htmlBody, textBody } = getEmailBody(emailType, replacements)
    const mailData = {
      from: config.get('mailer.from'),
      to: user.email,
      subject: 'Hindawi Invitation',
      text: textBody,
      html: htmlBody,
    }

    return Email.send(mailData)
  },
  setupAssignEmail: async (email, emailType, dashBoardUrl) => {
    let subject
    let replacements = {}
    switch (emailType) {
      case 'assign-handling-editor':
        subject = 'Hindawi Handling Editor Invitation'
        replacements = {
          url: dashBoardUrl,
        }
        break
      case 'add-author':
        subject = 'Manuscript Assignment on Hindawi'
        replacements = {
          url: dashBoardUrl,
        }
        break
      case 'invite-reviewer':
        subject = 'Review Request'
        replacements = {
          url: dashBoardUrl,
        }
        break
      default:
        subject = 'Welcome to Hindawi!'
        break
    }

    const { htmlBody, textBody } = getEmailBody(emailType, replacements)
    const mailData = {
      from: config.get('mailer.from'),
      to: email,
      subject,
      text: textBody,
      html: htmlBody,
    }
    return Email.send(mailData)
  },
  setupRevokeInvitationEmail: async (email, emailType) => {
    let subject
    const replacements = {}
    switch (emailType) {
      case 'revoke-handling-editor':
        subject = 'Invitation has been Cancelled'
        break
      case 'unassign-reviewer':
        subject = 'Reviewer Unassigned'
        break
      default:
        throw new Error('Invalid emailType')
    }

    const { htmlBody, textBody } = getEmailBody(emailType, replacements)
    const mailData = {
      from: config.get('mailer.from'),
      to: email,
      subject,
      text: textBody,
      html: htmlBody,
    }
    return Email.send(mailData)
  },
  setupAgreeEmail: async (toEmail, user, invRole, url, collectionId) => {
    let role = 'Handling Editor'
    let subject = 'Handling Editor Agreed'
    if (invRole === 'reviewer') {
      subject = 'Reviewer Agreed'
      role = 'Reviewer'
    }
    const { htmlBody, textBody } = getEmailBody('agree', {
      url,
      role,
      name: `${user.firstName} ${user.lastName}`,
      collectionId,
    })

    const mailData = {
      from: config.get('mailer.from'),
      to: toEmail,
      subject,
      text: textBody,
      html: htmlBody,
    }
    return Email.send(mailData)
  },
  setupDeclineEmail: async (toEmail, user, emailType, collectionId, reason) => {
    let finalReason = ''
    if (reason !== undefined) {
      finalReason = `Reason: "${reason}"`
    }
    const replacements = {
      finalReason,
      name: `${user.firstName} ${user.lastName}`,
      collectionId,
    }

    const { htmlBody, textBody } = getEmailBody(emailType, replacements)
    const mailData = {
      from: config.get('mailer.from'),
      to: toEmail,
      subject: 'Handling Editor Declined',
      text: textBody,
      html: htmlBody,
    }

    return Email.send(mailData)
  },
  setupReviewerInvitationEmail: async ({
    invitedUser,
    baseUrl,
    collection: { id: collectionId, authorName, handlingEditor },
    fragment: { title, id: fragmentId, abstract, authors },
    invitation: { id: invitationId, timestamp },
    resend,
    subject,
  }) => {
    let agreeUrl = `${baseUrl}/projects/${collectionId}/versions/${fragmentId}/details?${querystring.encode(
      {
        invitationId,
        agree: true,
      },
    )}`
    const declineUrl = `${baseUrl}${resetPasswordPath}?${querystring.encode({
      collectionId,
      agree: false,
      invitationId,
      invitationToken: invitedUser.invitationToken,
    })}`
    if (!invitedUser.isConfirmed) {
      agreeUrl = `${baseUrl}${resetPasswordPath}?${querystring.encode({
        email: invitedUser.email,
        token: invitedUser.passwordResetToken,
        collectionId,
        fragmentId,
        agree: true,
        invitationId,
      })}`
    }

    const daysExpected = resend ? 0 : 14

    const date = new Date(timestamp)
    let expectedDate = date.getDate() + daysExpected
    date.setDate(expectedDate)
    expectedDate = date.toLocaleDateString('en-US', {
      day: 'numeric',
      month: 'long',
      year: 'numeric',
    })

    const replacements = {
      reviewerName: `${invitedUser.firstName} ${invitedUser.lastName}`,
      agreeUrl,
      declineUrl,
      baseUrl,
      title,
      editorName: handlingEditor.name,
      submittingAuthorName: authorName,
      expectedDate,
      abstract,
      editorEmail: handlingEditor.email,
      authors,
    }
    const emailTemplate = resend ? 'resend-reviewer' : 'invite-reviewer'
    const { htmlBody, textBody } = getEmailBody(emailTemplate, replacements)
    const mailData = {
      from: config.get('mailer.from'),
      to: invitedUser.email,
      subject,
      text: textBody,
      html: htmlBody,
    }
    return Email.send(mailData)
  },
  setupReviewerUnassignEmail: async (
    invitedUser,
    collection,
    title,
    authorName,
  ) => {
    const subject = `${collection.customId}: Review Unassigned`

    const replacements = {
      reviewerName: `${invitedUser.firstName} ${invitedUser.lastName}`,
      title,
      editorName: collection.handlingEditor.name,
      authorName,
      staffEmail: config.get('mailer.from'),
    }

    const { htmlBody, textBody } = getEmailBody(
      'unassign-reviewer',
      replacements,
    )
    const mailData = {
      from: config.get('mailer.from'),
      to: invitedUser.email,
      subject,
      text: textBody,
      html: htmlBody,
    }
    return Email.send(mailData)
  },
  setupReviewerDeclinedEmail: async (
    toEmail,
    invitedUser,
    collection,
    title,
    authorName,
    fragmentId,
    eicName,
    baseUrl,
  ) => {
    const subject = `${collection.customId}: Manuscript Reviews`
    const manuscriptUrl = `${baseUrl}/projects/${
      collection.id
    }/versions/${fragmentId}/details`
    const replacements = {
      reviewerName: `${invitedUser.firstName} ${invitedUser.lastName}`,
      title,
      editorName: collection.handlingEditor.name,
      authorName,
      manuscriptUrl,
      eicName,
    }

    const { htmlBody, textBody } = getEmailBody(
      'reviewer-declined',
      replacements,
    )
    const mailData = {
      from: config.get('mailer.from'),
      to: toEmail,
      subject,
      text: textBody,
      html: htmlBody,
    }
    return Email.send(mailData)
  },
}

const getEmailBody = (emailType, replacements) => {
  const htmlFile = readFile(`${__dirname}/templates/${emailType}.html`)
  const textFile = readFile(`${__dirname}/templates/${emailType}.txt`)
  const htmlTemplate = handlebars.compile(htmlFile)
  const textTemplate = handlebars.compile(textFile)
  const htmlBody = htmlTemplate(replacements)
  const textBody = textTemplate(replacements)
  return { htmlBody, textBody }
}

const readFile = path =>
  fs.readFileSync(path, { encoding: 'utf-8' }, (err, file) => {
    if (err) {
      throw err
    } else {
      return file
    }
  })
